$ifndef _PWM_INC_
$define _PWM_INC_
'
' Hardware PWM manipulation procedures
'
' For use with PIC24F and PIC24H devices
'
'-----------------------------------------------------------------------------
' Enable OC1 operation
' Input     : None
' Output    : None
' Notes     : Makes the OC1 pin an output and enables OC1
'  
$define HPWM1_Enable() '
    OC1CON = %0000000000000110 '
    Output __CCP1_PIN        

'-----------------------------------------------------------------------------
' Disable OC1 operation
' Input     : None
' Output    : None
' Notes     : Makes the OC1 pin an input and disables OC1
'   
$define HPWM1_Disable() '
   Input __CCP1_PIN     '
   OC1CON = $00
   
'-----------------------------------------------------------------------------
' Enable OC2 operation
' Input     : None
' Output    : None
' Notes     : Makes the OC2 pin an output and enables OC2
'  
$define HPWM2_Enable() '
    OC2CON = %0000000000000110 '
    Output __CCP2_PIN       

'-----------------------------------------------------------------------------
' Disable OC2 operation
' Input     : None
' Output    : None
' Notes     : Makes the OC2 pin an input and disables OC2
'   
$define HPWM2_Disable() '
   Input __CCP2_PIN     '
   OC2CON = $00

$ifdef _OC3CON
'-----------------------------------------------------------------------------
' Enable OC3 operation
' Input     : None
' Output    : None
' Notes     : Makes the OC3 pin an output and enables OC3
'  
$define HPWM3_Enable() '
    OC3CON = %0000000000000110 '
    Output __CCP3_PIN       

'-----------------------------------------------------------------------------
' Disable OC3 operation
' Input     : None
' Output    : None
' Notes     : Makes the OC3 pin an input and disables OC3
'   
$define HPWM3_Disable() '
   Input __CCP3_PIN     '
   OC3CON = $00
   
$endif ' OC3CON

$ifdef _OC4CON
'-----------------------------------------------------------------------------
' Enable OC4 operation
' Input     : None
' Output    : None
' Notes     : Makes the OC4 pin an output and enables OC4
'  
$define HPWM4_Enable() '
    OC4CON = %0000000000000110 '
    Output __CCP4_PIN       

'-----------------------------------------------------------------------------
' Disable OC4 operation
' Input     : None
' Output    : None
' Notes     : Makes the OC4 pin an input and disables OC4
'   
$define HPWM4_Disable() '
   Input __CCP4_PIN     '
   OC4CON = $00
   
$endif ' OC4CON

$ifdef _OC5CON
'-----------------------------------------------------------------------------
' Enable OC5 operation
' Input     : None
' Output    : None
' Notes     : Makes the OC5 pin an output and enables OC5
'  
$define HPWM5_Enable() '
    OC5CON = %0000000000000110 '
    Output __CCP5_PIN       

'-----------------------------------------------------------------------------
' Disable OC5 operation
' Input     : None
' Output    : None
' Notes     : Makes the OC5 pin an input and disables OC5
'   
$define HPWM5_Disable() '
   Input __CCP5_PIN     '
   OC5CON = $00
   
$endif ' OC5CON
'-----------------------------------------------------------------------------
' Helper procedure to calculate a duty cycle required for a given PWM frequency
' Input     : PR2 holds the contents of the PR2 register
'           : pDuty holds the 8-bit duty cycle (0 to 255)
' Output    : Returns the 16-bit duty cycle value To be placed into an OCxRS SFR
' Notes     : None
'
Proc HPWM_mCalculateDuty(pDuty As WREG0.Byte0), WREG0   
'
' Create a 32-bit variable from W4:W5
'    
    Dim mWREG4 As WREG4.Byte0
    Dim mWREG4H As WREG4.Byte1
    Dim mWREG4HH As WREG5.Byte0
    Dim mWREG4HHH As WREG5.Byte1   
    Dim dWREG4 As mWREG4.Dword
    
    dWREG4 = PR2 * pDuty                           ' \
    Result = dWREG4 / 256                          ' / Calculate duty value and return it
EndProc 
'-----------------------------------------------------------------------------
' Helper procedure to calculate the values required for a given frequency and duty cycle 
' Input     : pFrequency holds the 16-bit frequency (in Hz)
'           : pDuty holds the 8-bit duty cycle (0 to 255) 
' Output    : Returns the 16-Bit duty cycle value To be placed into an OCxRS SFR
'           : SRbit_Cs will hold 1 if the frequency is achievable
' Notes     : None
'
Proc HPWM_mSetFreq(pFrequency As WREG6, pDuty As Byte), Word

    Dim wPRConst As Word   
    Dim bPreSelect As Byte
    Dim wPrescaleValue As Word       
'
' Create a 32-bit variable from W0:W1
'    
    Dim mWREG0 As WREG0.Byte0
    Dim mWREG0H As WREG0.Byte1
    Dim mWREG0HH As WREG1.Byte0
    Dim mWREG0HHH As WREG1.Byte1    
    Dim dWREG0 As mWREG0.Dword 
         
    Symbol cFosc = _xtal * 1000000
    
    dWREG0 = cFosc / pFrequency                         ' \
    wPRConst = dWREG0 / 2                               ' / Calculate the period for a given frequency        
    bPreSelect = 1                                      ' Reset the Prescale choosing variable
    wPrescaleValue = 1                                  ' Default to a 1:1 prescaler
    T2CON = %0000000000000000                           ' Disable Timer2 and set for a default prescaler of 1:1
    Repeat                                              ' Create a loop to find the correct prescaler for a given frequency
        WREG2 = wPRConst / wPrescaleValue               ' \
        WREG2 = WREG2 - 1                               ' / Calculate a PR2 value
        If WREG2 <= 512 Then                            ' \ 
            If WREG2 > 1 Then                           ' / Have we reached an acceptable value for PR2?
                PR2 = WREG2                             ' Yes. So load PR2 with it
                Result = HPWM_mCalculateDuty(pDuty)     ' Find the duty value for a given PWM period(also return it) 
                Set T2CONbits_TON                       ' Enable Timer2 
                Set SRbits_C                            ' Set the carry flag to signify success with frequency
                Return                                  ' Exit the procedure
            EndIf
        EndIf                                           ' Otherwise... We could not achieve the frequency so...
        If bPreSelect.0 = 1 Then                        ' Is bit-0 of bPreSelect set? 
            wPrescaleValue = 8                          ' Yes. So we are now using a prescaler of 1:8
            WREG2 = %0000000000010000                   ' Set T2CON for a prescaler of 1:8
        ElseIf bPreSelect.1 = 1 Then                    ' Otherwise... Is bit-1 of bPreSelect Set?
            wPrescaleValue = 64                         ' Yes. So we are now using a prescaler of 1:64
            WREG2 = %0000000000100000                   ' Set T2CON for a prescaler of 1:64
        Else                                            ' Otherwise... It must require a prescaler of 1:256.
            wPrescaleValue = 256                        ' So we are now using a prescaler of 1:64
            WREG2 = %0000000000110000                   ' Set T2CON for a prescaler of 1:256
        EndIf      
        T2CON = WREG2                                   ' Place WREG2 into T2CON
        bPreSelect = bPreSelect << 1                    ' Move the bits of the prescale select variable 1 to the left      
    Until bPreSelect.3 = 1                              ' Loop until all prescale values have been tried                           
    Clear SRbits_C                                      ' Clear the carry flag to signify the frequency could not be reached
EndProc

'-----------------------------------------------------------------------------
' Calculate the values required for a given frequency and duty cycle for OC1
' Input     : pFrequency holds the 16-bit frequency (in Hz)
'           : pDuty holds the 8-bit duty cycle (0 to 255) 
' Output    : Returns 1 as an indication to whether the frequency can be acheived
' Notes     : None
'
Proc HPWM1_SetFreq(pFrequency As Word, pDuty As Byte), Bit
    OC1RS = HPWM_mSetFreq(pFrequency, pDuty)    ' Calculate the value for a given frequency and duty cycle
    If SRbits_C = 1 Then                        ' Can the frequency be achieved?
        HPWM1_Enable()                          ' Yes. So enable PWM1
    EndIf
    Result = SRbits_C   
EndProc

'-----------------------------------------------------------------------------
' Adjust the duty cycle of OC1
' Input     : pDuty holds the 8-bit duty cycle (0 to 255)
' Output    : None
' Notes     : None
'
Proc HPWM1_Duty(pDuty As WREG0.Byte0)       
    OC1RS = HPWM_mCalculateDuty(pDuty)
EndProc
    
'-----------------------------------------------------------------------------
' Calculate the values required for a given frequency and duty cycle for OC2
' Input     : pFrequency holds the 16-bit frequency (in Hz)
'           : pDuty holds the 8-bit duty cycle (0 to 255) 
' Output    : Returns 1 as an indication to whether the frequency can be acheived
' Notes     : None
'
Proc HPWM2_SetFreq(pFrequency As Word, pDuty As Byte), Bit
    OC2RS = HPWM_mSetFreq(pFrequency, pDuty)    ' Calculate the value for a given frequency and duty cycle
    If SRbits_C = 1 Then                        ' Can the frequency be acheived?
        HPWM2_Enable()                          ' OC3RSnable PWM2                             
    EndIf
    Result = SRbits_C    
EndProc

'-----------------------------------------------------------------------------
' Adjust the duty cycle of OC2
' Input     : pDuty holds the 8-bit duty cycle (0 to 255)
' Output    : None
' Notes     : None
'
Proc HPWM2_Duty(pDuty As WREG0.Byte0)    
    OC2RS = HPWM_mCalculateDuty(pDuty)
EndProc

$ifdef _OC3CON
'-----------------------------------------------------------------------------
' Calculate the values required for a given frequency and duty cycle for OC3
' Input     : pFrequency holds the 16-bit frequency (in Hz)
'           : pDuty holds the 8-bit duty cycle (0 to 255) 
' Output    : Returns 1 as an indication to whether the frequency can be acheived
' Notes     : None
'
Proc HPWM3_SetFreq(pFrequency As Word, pDuty As Byte), Bit
    OC3RS = HPWM_mSetFreq(pFrequency, pDuty)    ' Calculate the value for a given frequency and duty cycle
    If SRbits_C = 1 Then                        ' Can the frequency be achieved?
        HPWM3_Enable()                          ' OC4RSnable PWM3                            
    EndIf 
    Result = SRbits_C  
EndProc

'-----------------------------------------------------------------------------
' Adjust the duty cycle of OC3
' Input     : pDuty holds the 8-bit duty cycle (0 to 255)
' Output    : None
' Notes     : None
'
Proc HPWM3_Duty(pDuty As WREG0.Byte0)
    OC3RS = HPWM_mCalculateDuty(pDuty)
EndProc

$endif  '_OC3CON
  
$ifdef _OC4CON
'-----------------------------------------------------------------------------
' Calculate the values required for a given frequency and duty cycle for OC4
' Input     : pFrequency holds the 16-bit frequency (in Hz)
'           : pDuty holds the 8-bit duty cycle (0 to 255) 
' Output    : Returns 1 as an indication to whether the frequency can be acheived
' Notes     : None
'
Proc HPWM4_SetFreq(pFrequency As Word, pDuty As Byte), Bit
    OC4RS = HPWM_mSetFreq(pFrequency, pDuty)    ' Calculate the value for a given frequency and duty cycle
    If SRbits_C = 1 Then                        ' Can the frequency be achieved?
        HPWM4_Enable()                          ' Yes. So enable PWM4                             
    EndIf
    Result = SRbits_C   
EndProc

'-----------------------------------------------------------------------------
' Adjust the duty cycle of OC4
' Input     : pDuty holds the 8-bit duty cycle (0 to 255)
' Output    : None
' Notes     : None
'
Proc HPWM4_Duty(pDuty As WREG0.Byte0)
    OC4RS = HPWM_mCalculateDuty(pDuty)
EndProc

$endif  '_OC4CON

$ifdef _OC5CON
'-----------------------------------------------------------------------------
' Calculate the values required for a given frequency and duty cycle for OC5
' Input     : pFrequency holds the 16-bit frequency (in Hz)
'           : pDuty holds the 8-bit duty cycle (0 to 255) 
' Output    : Returns 1 as an indication to whether the frequency can be acheived
' Notes     : None
'
Proc HPWM5_SetFreq(pFrequency As Word, pDuty As Byte), Bit
    OC5RS = HPWM_mSetFreq(pFrequency, pDuty)    ' Calculate the value for a given frequency and duty cycle
    If SRbits_C = 1 Then                        ' Can the frequency be achieved?
        HPWM5_Enable()                          ' Yes. So enable PWM5 
    EndIf
    Result = SRbits_C   
EndProc

'-----------------------------------------------------------------------------
' Adjust the duty cycle of OC5
' Input     : pDuty holds the 8-bit duty cycle (0 to 255)
' Output    : None
' Notes     : None
'
Proc HPWM5_Duty(pDuty As WREG0.Byte0)
    OC5RS = HPWM_mCalculateDuty(pDuty)
EndProc

$endif  '_OC5CON
$endif  ' _PWM_INC_          

